@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import json
+import copy
 from zipfile import  ZipFile
 from XmindToTestlink import preprocess
 
@@ -36,17 +37,39 @@ class XmindToDict(object):
     def get_summery(self, in_dict):
         return in_dict['plain']['content']
     
-    def get_custom_fields(self, in_data):
-        customs = []
+    #def get_custom_fields(self, old_custom, in_data):
+    #    customs = []
+    #    for attach in in_data['children']['attached']:
+    #        if isinstance(attach, dict):
+    #            custom = {}
+    #            if "children" in attach and attach['children'] != {}:
+    #                custom[attach['title']] = attach['children']['attached'][0]['title']
+    #            else:
+    #                custom[attach['title']] = ""
+    #            customs.append(custom)
+    #    return customs
+    def get_custom_fields(self, save_custom, in_data):
+        old_custom = copy.deepcopy(save_custom)
         for attach in in_data['children']['attached']:
+            find = 0
             if isinstance(attach, dict):
-                custom = {}
-                if "children" in attach and attach['children'] != {}:
-                    custom[attach['title']] = attach['children']['attached'][0]['title']
+                ol = len(old_custom)
+                if ol >= 1:
+                    for i in range(ol):
+                        for k,v in old_custom[i].items():
+                            if k == attach['title']:
+                                old_custom[i][k] = v + "|" + attach['children']['attached'][0]['title']
+                                find = 1
+                    if not find:
+                        custom = {}
+                        custom[attach['title']] = attach['children']['attached'][0]['title']
+                        old_custom.append(custom)
                 else:
-                    custom[attach['title']] = ""
-                customs.append(custom)
-        return customs
+                    old_custom = []
+                    custom = {}
+                    custom[attach['title']] = attach['children']['attached'][0]['title']
+                    old_custom.append(custom)
+        return old_custom
     
     def get_steps(self, in_data):
         steps = []
@@ -68,7 +91,7 @@ class XmindToDict(object):
         else:
             return 2
     
-    def create_case(self, in_data, pre_dict, num, req_spec, req):
+    def create_case(self, in_data, pre_dict, num, req_spec, req, save_custom = ''):
         case = {
                 'title':'',
                 'summary':'',
@@ -79,6 +102,7 @@ class XmindToDict(object):
                 'step':'',
                 'reqband':''
                 }
+        case['custom_field'] = save_custom
         if req != '' and req_spec != '':
             case['reqband'] = {req: req_spec}
         if req_spec != '' and req == '':
@@ -92,7 +116,7 @@ class XmindToDict(object):
                 for attach in in_data['children']['attached']:
                     if isinstance(attach, dict):
                         if attach['title'] == "属性" and "children" in attach and attach['children'] != {}:
-                            case['custom_field'] = self.get_custom_fields(attach)
+                            case['custom_field'] = self.get_custom_fields(save_custom, attach)
                             if "执行方式" in case['custom_field']:
                                 case['execution_type'] = self.get_execution_type(case['custom_field']['执行方式'])
                                 del case['custom_field']['执行方式']
@@ -131,7 +155,8 @@ class XmindToDict(object):
             return 0
     
     def is_case(self, in_data):
-        for branch in in_data['children']['attached']:
+        if len(in_data['children']['attached']) == 1: 
+            branch = in_data['children']['attached'][0]
             if branch['title'] == "属性" or branch['title'] == "步骤":
                 return 1
     
@@ -186,15 +211,17 @@ class XmindToDict(object):
                         if isinstance(attach, dict):
                             self.get_req_dict(attach, out_dict, start)
 
-    def get_dict(self, in_data, out_dict, start = 0, pre_dict = {}, count = 0, req_spec = '', req = ''):
-        def parse_xmind(in_data, out_dict, start = 0, pre_dict = {}, count = 0, req_spec = '', req = ''):
+    def get_dict(self, in_data, out_dict, start = 0, pre_dict = {}, count = 0, req_spec = '', req = '', save_custom = []):
+        def parse_xmind(in_data, out_dict, start = 0, pre_dict = {}, count = 0, req_spec = '', req = '', save_custom = []):
             if "title" in in_data.keys():
+                if in_data['title'] == "属性":
+                    return
                 if in_data['title'] == "用例":
                     start = 1
                 elif in_data['title'] != "用例":
                     sc = self.check_suite_or_case(in_data)
                     if sc == "case":
-                        case = self.create_case(in_data, pre_dict, count, req_spec, req)
+                        case = self.create_case(in_data, pre_dict, count, req_spec, req, save_custom)
                         out_dict['cases'].append(case)
                         return 1
                     elif sc == "suite":
@@ -231,19 +258,27 @@ class XmindToDict(object):
                             if pre['id'] == v:
                                 pre_dict[k] = pre['title']
                 if "attached" in in_data['children']:
+                    #检查继承属性
+                    save_custom = self.check_save_custom(save_custom, in_data['children']['attached'])
                     #递归，遍历嵌套的节点
                     for count in range(len(in_data['children']['attached'])):
                         attach = in_data['children']['attached'][count]
                         if isinstance(attach, dict):
-                            self.get_dict(attach, out_dict, start, pre_dict, count, req_spec, req)
+                            self.get_dict(attach, out_dict, start, pre_dict, count, req_spec, req, save_custom)
 
         if self.type == "case":
             if not self.have_case(in_data) and start == 0:
                 return 0
             else:
-                parse_xmind(in_data, out_dict, start, pre_dict, count, req_spec, req)
+                parse_xmind(in_data, out_dict, start, pre_dict, count, req_spec, req, save_custom)
         elif self.type == "req":
-            parse_xmind(in_data, out_dict, start, pre_dict, count)
+            parse_xmind(in_data, out_dict, start, pre_dict, count, req_spec, req, save_custom)
+
+    def check_save_custom(self, old_custom, in_data):
+        for attach in in_data: 
+            if "title" in attach.keys() and attach['title'] == "属性":
+                old_custom = self.get_custom_fields(old_custom, attach)
+        return old_custom
     
     def get_root(self, in_dict):
         root_name = in_dict[0]['title']
